@@ -86,6 +86,29 @@ func TestCompile_Success(t *testing.T) {
 		`SELECT elapsed(distinct(value)) FROM cpu WHERE time >= now() - 5m GROUP BY time(1m)`,
 		`SELECT cumulative_sum(distinct(value)) FROM cpu WHERE time >= now() - 5m GROUP BY time(1m)`,
 		`SELECT last(value) / (1 - 0) FROM cpu`,
+		`SELECT abs(value) FROM cpu`,
+		`SELECT sin(value) FROM cpu`,
+		`SELECT cos(value) FROM cpu`,
+		`SELECT tan(value) FROM cpu`,
+		`SELECT asin(value) FROM cpu`,
+		`SELECT acos(value) FROM cpu`,
+		`SELECT atan(value) FROM cpu`,
+		`SELECT sqrt(value) FROM cpu`,
+		`SELECT pow(value, 2) FROM cpu`,
+		`SELECT pow(value, 3.14) FROM cpu`,
+		`SELECT pow(2, value) FROM cpu`,
+		`SELECT pow(3.14, value) FROM cpu`,
+		`SELECT exp(value) FROM cpu`,
+		`SELECT atan2(value, 0.1) FROM cpu`,
+		`SELECT atan2(0.2, value) FROM cpu`,
+		`SELECT atan2(value, 1) FROM cpu`,
+		`SELECT atan2(2, value) FROM cpu`,
+		`SELECT ln(value) FROM cpu`,
+		`SELECT log(value, 2) FROM cpu`,
+		`SELECT log2(value) FROM cpu`,
+		`SELECT log10(value) FROM cpu`,
+		`SELECT sin(value) - sin(1.3) FROM cpu`,
+		`SELECT value FROM cpu WHERE sin(value) > 0.5`,
 	} {
 		t.Run(tt, func(t *testing.T) {
 			stmt, err := influxql.ParseStatement(tt)
@@ -322,6 +345,21 @@ func TestCompile_Failures(t *testing.T) {
 		{s: `SELECT value FROM myseries WHERE value OR time >= now() - 1m`, err: `invalid condition expression: value`},
 		{s: `SELECT value FROM myseries WHERE time >= now() - 1m OR value`, err: `invalid condition expression: value`},
 		{s: `SELECT value FROM (SELECT value FROM cpu ORDER BY time DESC) ORDER BY time ASC`, err: `subqueries must be ordered in the same direction as the query itself`},
+		{s: `SELECT sin(value, 3) FROM cpu`, err: `invalid number of arguments for sin, expected 1, got 2`},
+		{s: `SELECT cos(2.3, value, 3) FROM cpu`, err: `invalid number of arguments for cos, expected 1, got 3`},
+		{s: `SELECT tan(value, 3) FROM cpu`, err: `invalid number of arguments for tan, expected 1, got 2`},
+		{s: `SELECT asin(value, 3) FROM cpu`, err: `invalid number of arguments for asin, expected 1, got 2`},
+		{s: `SELECT acos(value, 3.2) FROM cpu`, err: `invalid number of arguments for acos, expected 1, got 2`},
+		{s: `SELECT atan() FROM cpu`, err: `invalid number of arguments for atan, expected 1, got 0`},
+		{s: `SELECT sqrt(42, 3, 4) FROM cpu`, err: `invalid number of arguments for sqrt, expected 1, got 3`},
+		{s: `SELECT abs(value, 3) FROM cpu`, err: `invalid number of arguments for abs, expected 1, got 2`},
+		{s: `SELECT ln(value, 3) FROM cpu`, err: `invalid number of arguments for ln, expected 1, got 2`},
+		{s: `SELECT log2(value, 3) FROM cpu`, err: `invalid number of arguments for log2, expected 1, got 2`},
+		{s: `SELECT log10(value, 3) FROM cpu`, err: `invalid number of arguments for log10, expected 1, got 2`},
+		{s: `SELECT pow(value, 3, 3) FROM cpu`, err: `invalid number of arguments for pow, expected 2, got 3`},
+		{s: `SELECT atan2(value, 3, 3) FROM cpu`, err: `invalid number of arguments for atan2, expected 2, got 3`},
+		{s: `SELECT sin(1.3) FROM cpu`, err: `field must contain at least one variable`},
+		{s: `SELECT nofunc(1.3) FROM cpu`, err: `undefined function nofunc()`},
 	} {
 		t.Run(tt.s, func(t *testing.T) {
 			stmt, err := influxql.ParseStatement(tt.s)
