@@ -7,7 +7,6 @@ import (
 	"fmt"
 	"io"
 	"regexp"
-	"sync"
 	"time"
 
 	"github.com/gogo/protobuf/proto"
@@ -404,175 +403,43 @@ func NewCloseInterruptIterator(input Iterator, closing <-chan struct{}) Iterator
 	}
 }
 
-// AuxIterator represents an iterator that can split off separate auxiliary iterators.
-type AuxIterator interface {
-	Iterator
+// IteratorScanner is used to scan the results of an iterator into a map.
+type IteratorScanner interface {
+	// Peek retrieves information about the next point. It returns a timestamp, the name, and the tags.
+	Peek() (int64, string, Tags)
 
-	// Auxilary iterator
-	Iterator(name string, typ influxql.DataType) Iterator
+	// ScanAt will take a time, name, and tags and scan the point that matches those into the map.
+	ScanAt(ts int64, name string, tags Tags, values map[string]interface{})
 
-	// Start starts writing to the created iterators.
-	Start()
+	// Stats returns the IteratorStats from the Iterator.
+	Stats() IteratorStats
 
-	// Backgrounds the iterator so that, when start is called, it will
-	// continuously read from the iterator.
-	Background()
+	// Err returns an error that was encountered while scanning.
+	Err() error
+
+	io.Closer
 }
 
-// NewAuxIterator returns a new instance of AuxIterator.
-func NewAuxIterator(input Iterator, opt IteratorOptions) AuxIterator {
+// SkipDefault is a sentinel value to tell the IteratorScanner to skip setting the
+// default value if none was present. This causes the map to use the previous value
+// if it was previously set.
+var SkipDefault = interface{}(0)
+
+// NewIteratorScanner produces an IteratorScanner for the Iterator.
+func NewIteratorScanner(input Iterator, keys []influxql.VarRef, defaultValue interface{}) IteratorScanner {
 	switch input := input.(type) {
 	case FloatIterator:
-		return newFloatAuxIterator(input, opt)
+		return newFloatIteratorScanner(input, keys, defaultValue)
 	case IntegerIterator:
-		return newIntegerAuxIterator(input, opt)
+		return newIntegerIteratorScanner(input, keys, defaultValue)
 	case UnsignedIterator:
-		return newUnsignedAuxIterator(input, opt)
+		return newUnsignedIteratorScanner(input, keys, defaultValue)
 	case StringIterator:
-		return newStringAuxIterator(input, opt)
+		return newStringIteratorScanner(input, keys, defaultValue)
 	case BooleanIterator:
-		return newBooleanAuxIterator(input, opt)
+		return newBooleanIteratorScanner(input, keys, defaultValue)
 	default:
-		panic(fmt.Sprintf("unsupported aux iterator type: %T", input))
-	}
-}
-
-// auxIteratorField represents an auxilary field within an AuxIterator.
-type auxIteratorField struct {
-	name string            // field name
-	typ  influxql.DataType // detected data type
-	itrs []Iterator        // auxillary iterators
-	mu   sync.Mutex
-	opt  IteratorOptions
-}
-
-func (f *auxIteratorField) append(itr Iterator) {
-	f.mu.Lock()
-	defer f.mu.Unlock()
-	f.itrs = append(f.itrs, itr)
-}
-
-func (f *auxIteratorField) close() {
-	f.mu.Lock()
-	defer f.mu.Unlock()
-	for _, itr := range f.itrs {
-		itr.Close()
-	}
-}
-
-type auxIteratorFields struct {
-	fields     []*auxIteratorField
-	dimensions []string
-}
-
-// newAuxIteratorFields returns a new instance of auxIteratorFields from a list of field names.
-func newAuxIteratorFields(opt IteratorOptions) *auxIteratorFields {
-	fields := make([]*auxIteratorField, len(opt.Aux))
-	for i, ref := range opt.Aux {
-		fields[i] = &auxIteratorField{name: ref.Val, typ: ref.Type, opt: opt}
-	}
-	return &auxIteratorFields{
-		fields:     fields,
-		dimensions: opt.GetDimensions(),
-	}
-}
-
-func (a *auxIteratorFields) close() {
-	for _, f := range a.fields {
-		f.close()
-	}
-}
-
-// iterator creates a new iterator for a named auxilary field.
-func (a *auxIteratorFields) iterator(name string, typ influxql.DataType) Iterator {
-	for _, f := range a.fields {
-		// Skip field if it's name doesn't match.
-		// Exit if no points were received by the iterator.
-		if f.name != name || (typ != influxql.Unknown && f.typ != typ) {
-			continue
-		}
-
-		// Create channel iterator by data type.
-		switch f.typ {
-		case influxql.Float:
-			itr := &floatChanIterator{cond: sync.NewCond(&sync.Mutex{})}
-			f.append(itr)
-			return itr
-		case influxql.Integer:
-			itr := &integerChanIterator{cond: sync.NewCond(&sync.Mutex{})}
-			f.append(itr)
-			return itr
-		case influxql.Unsigned:
-			itr := &unsignedChanIterator{cond: sync.NewCond(&sync.Mutex{})}
-			f.append(itr)
-			return itr
-		case influxql.String, influxql.Tag:
-			itr := &stringChanIterator{cond: sync.NewCond(&sync.Mutex{})}
-			f.append(itr)
-			return itr
-		case influxql.Boolean:
-			itr := &booleanChanIterator{cond: sync.NewCond(&sync.Mutex{})}
-			f.append(itr)
-			return itr
-		default:
-		}
-	}
-
-	return &nilFloatIterator{}
-}
-
-// send sends a point to all field iterators.
-func (a *auxIteratorFields) send(p Point) (ok bool) {
-	values := p.aux()
-	for i, f := range a.fields {
-		var v interface{}
-		if i < len(values) {
-			v = values[i]
-		}
-
-		tags := p.tags()
-		tags = tags.Subset(a.dimensions)
-
-		// Send new point for each aux iterator.
-		// Primitive pointers represent nil values.
-		for _, itr := range f.itrs {
-			switch itr := itr.(type) {
-			case *floatChanIterator:
-				ok = itr.setBuf(p.name(), tags, p.time(), v) || ok
-			case *integerChanIterator:
-				ok = itr.setBuf(p.name(), tags, p.time(), v) || ok
-			case *unsignedChanIterator:
-				ok = itr.setBuf(p.name(), tags, p.time(), v) || ok
-			case *stringChanIterator:
-				ok = itr.setBuf(p.name(), tags, p.time(), v) || ok
-			case *booleanChanIterator:
-				ok = itr.setBuf(p.name(), tags, p.time(), v) || ok
-			default:
-				panic(fmt.Sprintf("invalid aux itr type: %T", itr))
-			}
-		}
-	}
-	return ok
-}
-
-func (a *auxIteratorFields) sendError(err error) {
-	for _, f := range a.fields {
-		for _, itr := range f.itrs {
-			switch itr := itr.(type) {
-			case *floatChanIterator:
-				itr.setErr(err)
-			case *integerChanIterator:
-				itr.setErr(err)
-			case *unsignedChanIterator:
-				itr.setErr(err)
-			case *stringChanIterator:
-				itr.setErr(err)
-			case *booleanChanIterator:
-				itr.setErr(err)
-			default:
-				panic(fmt.Sprintf("invalid aux itr type: %T", itr))
-			}
-		}
+		panic(fmt.Sprintf("unsupported type for iterator scanner: %T", input))
 	}
 }
 
@@ -791,7 +658,6 @@ func newIteratorOptionsStmt(stmt *influxql.SelectStatement, sopt SelectOptions)
 	opt.Limit, opt.Offset = stmt.Limit, stmt.Offset
 	opt.SLimit, opt.SOffset = stmt.SLimit, stmt.SOffset
 	opt.MaxSeriesN = sopt.MaxSeriesN
-	opt.InterruptCh = sopt.InterruptCh
 	opt.Authorizer = sopt.Authorizer
 
 	return opt, nil
@@ -883,6 +749,15 @@ func (opt IteratorOptions) SeekTime() int64 {
 	return opt.EndTime
 }
 
+// StopTime returns the time the iterator should end at.
+// For ascending iterators this is the end time, for descending iterators it's the start time.
+func (opt IteratorOptions) StopTime() int64 {
+	if opt.Ascending {
+		return opt.EndTime
+	}
+	return opt.StartTime
+}
+
 // Window returns the time window [start,end) that t falls within.
 func (opt IteratorOptions) Window(t int64) (start, end int64) {
 	if opt.Interval.IsZero() {
@@ -1105,30 +980,12 @@ func encodeIteratorOptions(opt *IteratorOptions) *internal.IteratorOptions {
 
 	// Set condition, if set.
 	if opt.Condition != nil {
-		pb.Condition = proto.String(encodeConditionExpr(opt.Condition))
+		pb.Condition = proto.String(opt.Condition.String())
 	}
 
 	return pb
 }
 
-// encodeConditionExpr will encode the condition to a string and will also
-// fix the condition by adding parenthesis if the AST was improperly created.
-func encodeConditionExpr(expr influxql.Expr) string {
-	cond := influxql.RewriteExpr(influxql.CloneExpr(expr), func(expr influxql.Expr) influxql.Expr {
-		if expr, ok := expr.(*influxql.BinaryExpr); ok {
-			if e, ok := expr.LHS.(*influxql.BinaryExpr); ok && e.Op.Precedence() < expr.Op.Precedence() {
-				expr.LHS = &influxql.ParenExpr{Expr: expr.LHS}
-			}
-			if e, ok := expr.RHS.(*influxql.BinaryExpr); ok && e.Op.Precedence() <= expr.Op.Precedence() {
-				expr.RHS = &influxql.ParenExpr{Expr: expr.RHS}
-			}
-			return expr
-		}
-		return expr
-	})
-	return cond.String()
-}
-
 func decodeIteratorOptions(pb *internal.IteratorOptions) (*IteratorOptions, error) {
 	opt := &IteratorOptions{
 		Interval:   decodeInterval(pb.GetInterval()),
@@ -1250,50 +1107,6 @@ func decodeMeasurement(pb *internal.Measurement) (*influxql.Measurement, error)
 	return mm, nil
 }
 
-// selectInfo represents an object that stores info about select fields.
-type selectInfo struct {
-	calls map[*influxql.Call]struct{}
-	refs  map[*influxql.VarRef]struct{}
-}
-
-// newSelectInfo creates a object with call and var ref info from stmt.
-func newSelectInfo(stmt *influxql.SelectStatement) *selectInfo {
-	info := &selectInfo{
-		calls: make(map[*influxql.Call]struct{}),
-		refs:  make(map[*influxql.VarRef]struct{}),
-	}
-	influxql.Walk(info, stmt.Fields)
-	return info
-}
-
-func (v *selectInfo) Visit(n influxql.Node) influxql.Visitor {
-	switch n := n.(type) {
-	case *influxql.Call:
-		v.calls[n] = struct{}{}
-		return nil
-	case *influxql.VarRef:
-		v.refs[n] = struct{}{}
-		return nil
-	}
-	return v
-}
-
-// FindSelector returns a selector from the selectInfo. This will only
-// return a selector if the Call is a selector and it's the only function
-// in the selectInfo.
-func (v *selectInfo) FindSelector() *influxql.Call {
-	if len(v.calls) != 1 {
-		return nil
-	}
-
-	for s := range v.calls {
-		if influxql.IsSelector(s) {
-			return s
-		}
-	}
-	return nil
-}
-
 // Interval represents a repeating interval for a query.
 type Interval struct {
 	Duration time.Duration
@@ -1351,79 +1164,6 @@ func (itr *nilFloatReaderIterator) Close() error {
 }
 func (*nilFloatReaderIterator) Next() (*FloatPoint, error) { return nil, nil }
 
-// integerFloatTransformIterator executes a function to modify an existing point for every
-// output of the input iterator.
-type integerFloatTransformIterator struct {
-	input IntegerIterator
-	fn    integerFloatTransformFunc
-}
-
-// Stats returns stats from the input iterator.
-func (itr *integerFloatTransformIterator) Stats() IteratorStats { return itr.input.Stats() }
-
-// Close closes the iterator and all child iterators.
-func (itr *integerFloatTransformIterator) Close() error { return itr.input.Close() }
-
-// Next returns the minimum value for the next available interval.
-func (itr *integerFloatTransformIterator) Next() (*FloatPoint, error) {
-	p, err := itr.input.Next()
-	if err != nil {
-		return nil, err
-	} else if p != nil {
-		return itr.fn(p), nil
-	}
-	return nil, nil
-}
-
-// integerFloatTransformFunc creates or modifies a point.
-// The point passed in may be modified and returned rather than allocating a
-// new point if possible.
-type integerFloatTransformFunc func(p *IntegerPoint) *FloatPoint
-
-type integerFloatCastIterator struct {
-	input IntegerIterator
-	point FloatPoint
-}
-
-func (itr *integerFloatCastIterator) Stats() IteratorStats { return itr.input.Stats() }
-func (itr *integerFloatCastIterator) Close() error         { return itr.input.Close() }
-func (itr *integerFloatCastIterator) Next() (*FloatPoint, error) {
-	p, err := itr.input.Next()
-	if p == nil || err != nil {
-		return nil, err
-	}
-
-	itr.point.Name = p.Name
-	itr.point.Tags = p.Tags
-	itr.point.Time = p.Time
-	itr.point.Nil = p.Nil
-	itr.point.Value = float64(p.Value)
-	itr.point.Aux = p.Aux
-	return &itr.point, nil
-}
-
-type unsignedFloatCastIterator struct {
-	input UnsignedIterator
-	point FloatPoint
-}
-
-func (itr *unsignedFloatCastIterator) Stats() IteratorStats { return itr.input.Stats() }
-func (itr *unsignedFloatCastIterator) Close() error         { return itr.input.Close() }
-func (itr *unsignedFloatCastIterator) Next() (*FloatPoint, error) {
-	p, err := itr.input.Next()
-	if p == nil || err != nil {
-		return nil, err
-	}
-
-	itr.point.Name = p.Name
-	itr.point.Tags = p.Tags
-	itr.point.Time = p.Time
-	itr.point.Nil = p.Nil
-	itr.point.Value = float64(p.Value)
-	itr.point.Aux = p.Aux
-	return &itr.point, nil
-}
-
 // IteratorStats represents statistics about an iterator.
 // Some statistics are available immediately upon iterator creation while
 // some are derived as the iterator processes data.
