@@ -3,6 +3,7 @@ package httpd
 import (
 	"bytes"
 	"compress/gzip"
+	"context"
 	"encoding/json"
 	"errors"
 	"expvar"
@@ -32,6 +33,7 @@ import (
 	"github.com/influxdata/influxdb/prometheus/remote"
 	"github.com/influxdata/influxdb/query"
 	"github.com/influxdata/influxdb/services/meta"
+	"github.com/influxdata/influxdb/services/storage"
 	"github.com/influxdata/influxdb/tsdb"
 	"github.com/influxdata/influxdb/uuid"
 	"github.com/influxdata/influxql"
@@ -97,7 +99,7 @@ type Handler struct {
 		AuthorizeWrite(username, database string) error
 	}
 
-	QueryExecutor *query.QueryExecutor
+	QueryExecutor *query.Executor
 
 	Monitor interface {
 		Statistics(tags map[string]string) ([]*monitor.Statistic, error)
@@ -108,6 +110,8 @@ type Handler struct {
 		WritePoints(database, retentionPolicy string, consistencyLevel models.ConsistencyLevel, user meta.User, points []models.Point) error
 	}
 
+	Store Store
+
 	Config    *Config
 	Logger    *zap.Logger
 	CLFLogger *log.Logger
@@ -115,6 +119,7 @@ type Handler struct {
 	stats     *Statistics
 
 	requestTracker *RequestTracker
+	writeThrottler *Throttler
 }
 
 // NewHandler returns a new instance of handler with routes.
@@ -124,10 +129,21 @@ func NewHandler(c Config) *Handler {
 		Config:         &c,
 		Logger:         zap.NewNop(),
 		CLFLogger:      log.New(os.Stderr, "[httpd] ", 0),
+		Store:          storage.NewStore(),
 		stats:          &Statistics{},
 		requestTracker: NewRequestTracker(),
 	}
 
+	// Limit the number of concurrent & enqueued write requests.
+	h.writeThrottler = NewThrottler(c.MaxConcurrentWriteLimit, c.MaxEnqueuedWriteLimit)
+	h.writeThrottler.EnqueueTimeout = c.EnqueuedWriteTimeout
+
+	// Disable the write log if they have been suppressed.
+	writeLogEnabled := c.LogEnabled
+	if c.SuppressWriteLog {
+		writeLogEnabled = false
+	}
+
 	h.AddRoutes([]Route{
 		Route{
 			"query-options", // Satisfy CORS checks.
@@ -147,7 +163,7 @@ func NewHandler(c Config) *Handler {
 		},
 		Route{
 			"write", // Data-ingest route.
-			"POST", "/write", true, true, h.serveWrite,
+			"POST", "/write", true, writeLogEnabled, h.serveWrite,
 		},
 		Route{
 			"prometheus-write", // Prometheus remote write
@@ -279,6 +295,15 @@ func (h *Handler) AddRoutes(routes ...Route) {
 			handler = http.HandlerFunc(hf)
 		}
 
+		// Throttle route if this is a write endpoint.
+		if r.Method == http.MethodPost {
+			switch r.Pattern {
+			case "/write", "/api/v1/prom/write":
+				handler = h.writeThrottler.Handler(handler)
+			default:
+			}
+		}
+
 		handler = h.responseWriter(handler)
 		if r.Gzipped {
 			handler = gzipFilter(handler)
@@ -442,10 +467,11 @@ func (h *Handler) serveQuery(w http.ResponseWriter, r *http.Request, user meta.U
 	async := r.FormValue("async") == "true"
 
 	opts := query.ExecutionOptions{
-		Database:  db,
-		ChunkSize: chunkSize,
-		ReadOnly:  r.Method == "GET",
-		NodeID:    nodeID,
+		Database:        db,
+		RetentionPolicy: r.FormValue("rp"),
+		ChunkSize:       chunkSize,
+		ReadOnly:        r.Method == "GET",
+		NodeID:          nodeID,
 	}
 
 	if h.Config.AuthEnabled {
@@ -946,8 +972,8 @@ func (h *Handler) servePromWrite(w http.ResponseWriter, r *http.Request, user me
 	h.writeHeader(w, http.StatusNoContent)
 }
 
-// servePromRead will convert a Prometheus remote read request into an InfluxQL query and
-// return data in Prometheus remote read protobuf format.
+// servePromRead will convert a Prometheus remote read request into a storage
+// query and returns data in Prometheus remote read protobuf format.
 func (h *Handler) servePromRead(w http.ResponseWriter, r *http.Request, user meta.User) {
 	compressed, err := ioutil.ReadAll(r.Body)
 	if err != nil {
@@ -969,105 +995,82 @@ func (h *Handler) servePromRead(w http.ResponseWriter, r *http.Request, user met
 
 	// Query the DB and create a ReadResponse for Prometheus
 	db := r.FormValue("db")
-	q, err := prometheus.ReadRequestToInfluxQLQuery(&req, db, r.FormValue("rp"))
+	rp := r.FormValue("rp")
+
+	readRequest, err := prometheus.ReadRequestToInfluxStorageRequest(&req, db, rp)
 	if err != nil {
 		h.httpError(w, err.Error(), http.StatusBadRequest)
 		return
 	}
 
-	// Check authorization.
-	if h.Config.AuthEnabled {
-		if err := h.QueryAuthorizer.AuthorizeQuery(user, q, db); err != nil {
-			if err, ok := err.(meta.ErrAuthorize); ok {
-				h.Logger.Info("Unauthorized request",
-					zap.String("user", err.User),
-					zap.Stringer("query", err.Query),
-					logger.Database(err.Database))
-			}
-			h.httpError(w, "error authorizing query: "+err.Error(), http.StatusForbidden)
-			return
-		}
+	ctx := context.Background()
+	rs, err := h.Store.Read(ctx, readRequest)
+	if err != nil {
+		h.httpError(w, err.Error(), http.StatusBadRequest)
+		return
 	}
-
-	opts := query.ExecutionOptions{
-		Database:  db,
-		ChunkSize: DefaultChunkSize,
-		ReadOnly:  true,
-	}
-
-	if h.Config.AuthEnabled {
-		// The current user determines the authorized actions.
-		opts.Authorizer = user
-	} else {
-		// Auth is disabled, so allow everything.
-		opts.Authorizer = query.OpenAuthorizer
-	}
-
-	// Make sure if the client disconnects we signal the query to abort
-	closing := make(chan struct{})
-	if notifier, ok := w.(http.CloseNotifier); ok {
-		// CloseNotify() is not guaranteed to send a notification when the query
-		// is closed. Use this channel to signal that the query is finished to
-		// prevent lingering goroutines that may be stuck.
-		done := make(chan struct{})
-		defer close(done)
-
-		notify := notifier.CloseNotify()
-		go func() {
-			// Wait for either the request to finish
-			// or for the client to disconnect
-			select {
-			case <-done:
-			case <-notify:
-				close(closing)
-			}
-		}()
-		opts.AbortCh = done
-	} else {
-		defer close(closing)
-	}
-
-	// Execute query.
-	results := h.QueryExecutor.ExecuteQuery(q, opts, closing)
+	defer rs.Close()
 
 	resp := &remote.ReadResponse{
 		Results: []*remote.QueryResult{{}},
 	}
-
-	// pull all results from the channel
-	for r := range results {
-		// Ignore nil results.
-		if r == nil {
+	for rs.Next() {
+		cur := rs.Cursor()
+		if cur == nil {
+			// no data for series key + field combination
 			continue
 		}
 
-		// read the series data and convert into Prometheus samples
-		for _, s := range r.Series {
-			ts := &remote.TimeSeries{
-				Labels: prometheus.TagsToLabelPairs(s.Tags),
+		tags := prometheus.RemoveInfluxSystemTags(rs.Tags())
+		var unsupportedCursor string
+		switch cur := cur.(type) {
+		case tsdb.FloatBatchCursor:
+			var series *remote.TimeSeries
+			for {
+				ts, vs := cur.Next()
+				if len(ts) == 0 {
+					break
+				}
+
+				// We have some data for this series.
+				if series == nil {
+					series = &remote.TimeSeries{
+						Labels: prometheus.ModelTagsToLabelPairs(tags),
+					}
+				}
+
+				for i, ts := range ts {
+					series.Samples = append(series.Samples, &remote.Sample{
+						TimestampMs: ts / int64(time.Millisecond),
+						Value:       vs[i],
+					})
+				}
 			}
 
-			for _, v := range s.Values {
-				t, ok := v[0].(time.Time)
-				if !ok {
-					h.httpError(w, fmt.Sprintf("value %v wasn't a time", v[0]), http.StatusBadRequest)
-					return
-				}
-				val, ok := v[1].(float64)
-				if !ok {
-					h.httpError(w, fmt.Sprintf("value %v wasn't a float64", v[1]), http.StatusBadRequest)
-				}
-				timestamp := t.UnixNano() / int64(time.Millisecond) / int64(time.Nanosecond)
-				ts.Samples = append(ts.Samples, &remote.Sample{
-					TimestampMs: timestamp,
-					Value:       val,
-				})
+			// There was data for the series.
+			if series != nil {
+				resp.Results[0].Timeseries = append(resp.Results[0].Timeseries, series)
 			}
+		case tsdb.IntegerBatchCursor:
+			unsupportedCursor = "int64"
+		case tsdb.UnsignedBatchCursor:
+			unsupportedCursor = "uint"
+		case tsdb.BooleanBatchCursor:
+			unsupportedCursor = "bool"
+		case tsdb.StringBatchCursor:
+			unsupportedCursor = "string"
+		default:
+			panic(fmt.Sprintf("unreachable: %T", cur))
+		}
+		cur.Close()
 
-			resp.Results[0].Timeseries = append(resp.Results[0].Timeseries, ts)
+		if len(unsupportedCursor) > 0 {
+			h.Logger.Info("Prometheus can't read cursor",
+				zap.String("cursor_type", unsupportedCursor),
+				zap.Stringer("series", tags),
+			)
 		}
 	}
-
 	data, err := proto.Marshal(resp)
 	if err != nil {
 		h.httpError(w, err.Error(), http.StatusInternalServerError)
@@ -1586,6 +1589,12 @@ func (h *Handler) recovery(inner http.Handler, name string) http.Handler {
 	})
 }
 
+// Store describes the behaviour of the storage packages Store type.
+type Store interface {
+	Read(ctx context.Context, req *storage.ReadRequest) (storage.Results, error)
+	WithLogger(log *zap.Logger)
+}
+
 // Response represents a list of statement results.
 type Response struct {
 	Results []*query.Result
@@ -1640,3 +1649,77 @@ func (r *Response) Error() error {
 	}
 	return nil
 }
+
+// Throttler represents an HTTP throttler that limits the number of concurrent
+// requests being processed as well as the number of enqueued requests.
+type Throttler struct {
+	current  chan struct{}
+	enqueued chan struct{}
+
+	// Maximum amount of time requests can wait in queue.
+	// Must be set before adding middleware.
+	EnqueueTimeout time.Duration
+
+	Logger *zap.Logger
+}
+
+// NewThrottler returns a new instance of Throttler that limits to concurrentN.
+// requests processed at a time and maxEnqueueN requests waiting to be processed.
+func NewThrottler(concurrentN, maxEnqueueN int) *Throttler {
+	return &Throttler{
+		current:  make(chan struct{}, concurrentN),
+		enqueued: make(chan struct{}, concurrentN+maxEnqueueN),
+		Logger:   zap.NewNop(),
+	}
+}
+
+// Handler wraps h in a middleware handler that throttles requests.
+func (t *Throttler) Handler(h http.Handler) http.Handler {
+	timeout := t.EnqueueTimeout
+
+	// Return original handler if concurrent requests is zero.
+	if cap(t.current) == 0 {
+		return h
+	}
+
+	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		// Start a timer to limit enqueued request times.
+		var timerCh <-chan time.Time
+		if timeout > 0 {
+			timer := time.NewTimer(timeout)
+			defer timer.Stop()
+			timerCh = timer.C
+		}
+
+		// Wait for a spot in the queue.
+		if cap(t.enqueued) > cap(t.current) {
+			select {
+			case t.enqueued <- struct{}{}:
+				defer func() { <-t.enqueued }()
+			default:
+				t.Logger.Warn("request throttled, queue full", zap.Duration("d", timeout))
+				http.Error(w, "request throttled, queue full", http.StatusServiceUnavailable)
+				return
+			}
+		}
+
+		// First check if we can immediately send in to current because there is
+		// available capacity. This helps reduce racyness in tests.
+		select {
+		case t.current <- struct{}{}:
+		default:
+			// Wait for a spot in the list of concurrent requests, but allow checking the timeout.
+			select {
+			case t.current <- struct{}{}:
+			case <-timerCh:
+				t.Logger.Warn("request throttled, exceeds timeout", zap.Duration("d", timeout))
+				http.Error(w, "request throttled, exceeds timeout", http.StatusServiceUnavailable)
+				return
+			}
+		}
+		defer func() { <-t.current }()
+
+		// Execute request.
+		h.ServeHTTP(w, r)
+	})
+}
