@@ -26,6 +26,7 @@ import (
 	"github.com/influxdata/influxdb/pkg/estimator"
 	"github.com/influxdata/influxdb/pkg/limiter"
 	"github.com/influxdata/influxdb/pkg/metrics"
+	"github.com/influxdata/influxdb/pkg/radix"
 	intar "github.com/influxdata/influxdb/pkg/tar"
 	"github.com/influxdata/influxdb/pkg/tracing"
 	"github.com/influxdata/influxdb/query"
@@ -37,8 +38,7 @@ import (
 	"go.uber.org/zap"
 )
 
-//go:generate tmpl -data=@iterator.gen.go.tmpldata iterator.gen.go.tmpl
-//go:generate tmpl -data=@iterator.gen.go.tmpldata batch_cursor.gen.go.tmpl
+//go:generate tmpl -data=@iterator.gen.go.tmpldata iterator.gen.go.tmpl batch_cursor.gen.go.tmpl cursor_iterator.gen.go.tmpl
 //go:generate tmpl -data=@file_store.gen.go.tmpldata file_store.gen.go.tmpl
 //go:generate tmpl -data=@encoding.gen.go.tmpldata encoding.gen.go.tmpl
 //go:generate tmpl -data=@compact.gen.go.tmpldata compact.gen.go.tmpl
@@ -146,7 +146,6 @@ type Engine struct {
 	snapWG   *sync.WaitGroup // waitgroup for running snapshot compactions
 
 	id           uint64
-	database     string
 	path         string
 	sfile        *tsdb.SeriesFile
 	logger       *zap.Logger // Logger to be used for important messages
@@ -163,7 +162,7 @@ type Engine struct {
 
 	MaxPointsPerBlock int
 
-	// CacheFlushMemorySizeThreshold specifies the minimum size threshodl for
+	// CacheFlushMemorySizeThreshold specifies the minimum size threshold for
 	// the cache when the engine should write a snapshot to a TSM file
 	CacheFlushMemorySizeThreshold uint64
 
@@ -172,6 +171,13 @@ type Engine struct {
 	// a snapshot of the cache to a TSM file
 	CacheFlushWriteColdDuration time.Duration
 
+	// WALEnabled determines whether writes to the WAL are enabled.  If this is false,
+	// writes will only exist in the cache and can be lost if a snapshot has not occurred.
+	WALEnabled bool
+
+	// Invoked when creating a backup file "as new".
+	GenerateFormatFileNameFunc func() FormatFileNameFunc
+
 	// Controls whether to enabled compactions when the engine is open
 	enableCompactionsOnOpen bool
 
@@ -184,27 +190,43 @@ type Engine struct {
 
 	// provides access to the total set of series IDs
 	seriesIDSets tsdb.SeriesIDSets
+
+	// seriesTypeMap maps a series key to field type
+	seriesTypeMap *radix.Tree
 }
 
 // NewEngine returns a new instance of Engine.
-func NewEngine(id uint64, idx tsdb.Index, database, path string, walPath string, sfile *tsdb.SeriesFile, opt tsdb.EngineOptions) tsdb.Engine {
-	w := NewWAL(walPath)
-	w.syncDelay = time.Duration(opt.Config.WALFsyncDelay)
+func NewEngine(id uint64, idx tsdb.Index, path string, walPath string, sfile *tsdb.SeriesFile, opt tsdb.EngineOptions) tsdb.Engine {
+	var wal *WAL
+	if opt.WALEnabled {
+		wal = NewWAL(walPath)
+		wal.syncDelay = time.Duration(opt.Config.WALFsyncDelay)
+	}
 
 	fs := NewFileStore(path)
-	cache := NewCache(uint64(opt.Config.CacheMaxMemorySize), path)
+	fs.openLimiter = opt.OpenLimiter
+	if opt.FileStoreObserver != nil {
+		fs.WithObserver(opt.FileStoreObserver)
+	}
+	fs.tsmMMAPWillNeed = opt.Config.TSMWillNeed
 
-	c := &Compactor{
-		Dir:       path,
-		FileStore: fs,
-		RateLimit: opt.CompactionThroughputLimiter,
+	cache := NewCache(uint64(opt.Config.CacheMaxMemorySize))
+
+	c := NewCompactor()
+	c.Dir = path
+	c.FileStore = fs
+	c.RateLimit = opt.CompactionThroughputLimiter
+
+	var planner CompactionPlanner = NewDefaultPlanner(fs, time.Duration(opt.Config.CompactFullWriteColdDuration))
+	if opt.CompactionPlannerCreator != nil {
+		planner = opt.CompactionPlannerCreator(opt.Config).(CompactionPlanner)
+		planner.SetFileStore(fs)
 	}
 
 	logger := zap.NewNop()
 	stats := &EngineStatistics{}
 	e := &Engine{
 		id:           id,
-		database:     database,
 		path:         path,
 		index:        idx,
 		sfile:        sfile,
@@ -212,25 +234,35 @@ func NewEngine(id uint64, idx tsdb.Index, database, path string, walPath string,
 		traceLogger:  logger,
 		traceLogging: opt.Config.TraceLoggingEnabled,
 
-		WAL:   w,
+		WAL:   wal,
 		Cache: cache,
 
 		FileStore:      fs,
 		Compactor:      c,
-		CompactionPlan: NewDefaultPlanner(fs, time.Duration(opt.Config.CompactFullWriteColdDuration)),
+		CompactionPlan: planner,
 
 		CacheFlushMemorySizeThreshold: uint64(opt.Config.CacheSnapshotMemorySize),
 		CacheFlushWriteColdDuration:   time.Duration(opt.Config.CacheSnapshotWriteColdDuration),
 		enableCompactionsOnOpen:       true,
+		WALEnabled:                    opt.WALEnabled,
+		GenerateFormatFileNameFunc:    func() FormatFileNameFunc { return DefaultFormatFileName },
 		stats:             stats,
 		compactionLimiter: opt.CompactionLimiter,
 		scheduler:         newScheduler(stats, opt.CompactionLimiter.Capacity()),
 		seriesIDSets:      opt.SeriesIDSets,
 	}
 
+	// Feature flag to enable per-series type checking, by default this is off and
+	// e.seriesTypeMap will be nil.
+	if os.Getenv("INFLUXDB_SERIES_TYPE_CHECK_ENABLED") != "" {
+		e.seriesTypeMap = radix.New()
+	}
+
 	if e.traceLogging {
 		fs.enableTraceLogging(true)
-		w.enableTraceLogging(true)
+		if e.WALEnabled {
+			e.WAL.enableTraceLogging(true)
+		}
 	}
 
 	return e
@@ -238,44 +270,59 @@ func NewEngine(id uint64, idx tsdb.Index, database, path string, walPath string,
 
 // Digest returns a reader for the shard's digest.
 func (e *Engine) Digest() (io.ReadCloser, int64, error) {
-	digestPath := filepath.Join(e.path, "digest.tsd")
+	log, logEnd := logger.NewOperation(e.logger, "Engine digest", "tsm1_digest")
+	defer logEnd()
 
-	// See if there's an existing digest file on disk.
-	f, err := os.Open(digestPath)
-	if err == nil {
-		// There is an existing digest file. Now see if it is still fresh.
-		fi, err := f.Stat()
-		if err != nil {
-			f.Close()
-			return nil, 0, err
-		}
+	log.Info("Starting digest", zap.String("tsm1_path", e.path))
 
-		if !e.LastModified().After(fi.ModTime()) {
-			// Existing digest is still fresh so return a reader for it.
+	digestPath := filepath.Join(e.path, DigestFilename)
+
+	// Get a list of tsm file paths from the FileStore.
+	files := e.FileStore.Files()
+	tsmfiles := make([]string, 0, len(files))
+	for _, f := range files {
+		tsmfiles = append(tsmfiles, f.Path())
+	}
+
+	// See if there's a fresh digest cached on disk.
+	fresh, reason := DigestFresh(e.path, tsmfiles, e.LastModified())
+	if fresh {
+		f, err := os.Open(digestPath)
+		if err == nil {
 			fi, err := f.Stat()
 			if err != nil {
-				f.Close()
+				log.Info("Digest aborted, couldn't stat digest file", logger.Shard(e.id), zap.Error(err))
 				return nil, 0, err
 			}
+
+			log.Info("Digest is fresh", logger.Shard(e.id), zap.String("path", digestPath))
+
+			// Return the cached digest.
 			return f, fi.Size(), nil
 		}
-
-		if err := f.Close(); err != nil {
-			return nil, 0, err
-		}
 	}
 
+	log.Info("Digest stale", logger.Shard(e.id), zap.String("reason", reason))
+
 	// Either no digest existed or the existing one was stale
 	// so generate a new digest.
 
+	// Make sure the directory exists, in case it was deleted for some reason.
+	if err := os.MkdirAll(e.path, 0777); err != nil {
+		log.Info("Digest aborted, problem creating shard directory path", zap.Error(err))
+		return nil, 0, err
+	}
+
 	// Create a tmp file to write the digest to.
 	tf, err := os.Create(digestPath + ".tmp")
 	if err != nil {
+		log.Info("Digest aborted, problem creating tmp digest", zap.Error(err))
 		return nil, 0, err
 	}
 
 	// Write the new digest to the tmp file.
-	if err := Digest(e.path, tf); err != nil {
+	if err := Digest(e.path, tsmfiles, tf); err != nil {
+		log.Info("Digest aborted, problem writing tmp digest", zap.Error(err))
 		tf.Close()
 		os.Remove(tf.Name())
 		return nil, 0, err
@@ -283,21 +330,26 @@ func (e *Engine) Digest() (io.ReadCloser, int64, error) {
 
 	// Rename the temporary digest file to the actual digest file.
 	if err := renameFile(tf.Name(), digestPath); err != nil {
+		log.Info("Digest aborted, problem renaming tmp digest", zap.Error(err))
 		return nil, 0, err
 	}
 
 	// Create and return a reader for the new digest file.
-	f, err = os.Open(digestPath)
+	f, err := os.Open(digestPath)
 	if err != nil {
+		log.Info("Digest aborted, opening new digest", zap.Error(err))
 		return nil, 0, err
 	}
 
 	fi, err := f.Stat()
 	if err != nil {
+		log.Info("Digest aborted, can't stat new digest", zap.Error(err))
 		f.Close()
 		return nil, 0, err
 	}
 
+	log.Info("Digest written", zap.String("tsm1_digest_path", digestPath), zap.Int64("size", fi.Size()))
+
 	return f, fi.Size(), nil
 }
 
@@ -549,11 +601,10 @@ func (e *Engine) SeriesSketches() (estimator.Sketch, estimator.Sketch, error) {
 
 // LastModified returns the time when this shard was last modified.
 func (e *Engine) LastModified() time.Time {
-	walTime := e.WAL.LastWriteTime()
 	fsTime := e.FileStore.LastModified()
 
-	if walTime.After(fsTime) {
-		return walTime
+	if e.WALEnabled && e.WAL.LastWriteTime().After(fsTime) {
+		return e.WAL.LastWriteTime()
 	}
 
 	return fsTime
@@ -631,13 +682,19 @@ func (e *Engine) Statistics(tags map[string]string) []models.Statistic {
 
 	statistics = append(statistics, e.Cache.Statistics(tags)...)
 	statistics = append(statistics, e.FileStore.Statistics(tags)...)
-	statistics = append(statistics, e.WAL.Statistics(tags)...)
+	if e.WALEnabled {
+		statistics = append(statistics, e.WAL.Statistics(tags)...)
+	}
 	return statistics
 }
 
 // DiskSize returns the total size in bytes of all TSM and WAL segments on disk.
 func (e *Engine) DiskSize() int64 {
-	return e.FileStore.DiskSizeBytes() + e.WAL.DiskSizeBytes()
+	var walDiskSizeBytes int64
+	if e.WALEnabled {
+		walDiskSizeBytes = e.WAL.DiskSizeBytes()
+	}
+	return e.FileStore.DiskSizeBytes() + walDiskSizeBytes
 }
 
 // Open opens and initializes the engine.
@@ -661,16 +718,20 @@ func (e *Engine) Open() error {
 
 	e.index.SetFieldSet(fields)
 
-	if err := e.WAL.Open(); err != nil {
-		return err
+	if e.WALEnabled {
+		if err := e.WAL.Open(); err != nil {
+			return err
+		}
 	}
 
 	if err := e.FileStore.Open(); err != nil {
 		return err
 	}
 
-	if err := e.reloadCache(); err != nil {
-		return err
+	if e.WALEnabled {
+		if err := e.reloadCache(); err != nil {
+			return err
+		}
 	}
 
 	e.Compactor.Open()
@@ -694,7 +755,10 @@ func (e *Engine) Close() error {
 	if err := e.FileStore.Close(); err != nil {
 		return err
 	}
-	return e.WAL.Close()
+	if e.WALEnabled {
+		return e.WAL.Close()
+	}
+	return nil
 }
 
 // WithLogger sets the logger for the engine.
@@ -705,7 +769,9 @@ func (e *Engine) WithLogger(log *zap.Logger) {
 		e.traceLogger = e.logger
 	}
 
-	e.WAL.WithLogger(e.logger)
+	if e.WALEnabled {
+		e.WAL.WithLogger(e.logger)
+	}
 	e.FileStore.WithLogger(e.logger)
 }
 
@@ -1014,7 +1080,13 @@ func (e *Engine) overlay(r io.Reader, basePath string, asNew bool) error {
 	}
 
 	// Load any new series keys to the index
-	readers := make([]chan seriesKey, 0, len(newFiles))
+	tsmFiles := make([]TSMFile, 0, len(newFiles))
+	defer func() {
+		for _, r := range tsmFiles {
+			r.Close()
+		}
+	}()
+
 	ext := fmt.Sprintf(".%s", TmpTSMFileExtension)
 	for _, f := range newFiles {
 		// If asNew is true, the files created from readFileFromBackup will be new ones
@@ -1025,9 +1097,6 @@ func (e *Engine) overlay(r io.Reader, basePath string, asNew bool) error {
 			continue
 		}
 
-		ch := make(chan seriesKey, 1)
-		readers = append(readers, ch)
-
 		fd, err := os.Open(f)
 		if err != nil {
 			return err
@@ -1037,30 +1106,23 @@ func (e *Engine) overlay(r io.Reader, basePath string, asNew bool) error {
 		if err != nil {
 			return err
 		}
-		defer r.Close()
-
-		go func(c chan seriesKey, r *TSMReader) {
-			n := r.KeyCount()
-			for i := 0; i < n; i++ {
-				key, typ := r.KeyAt(i)
-				c <- seriesKey{key, typ}
-			}
-			close(c)
-		}(ch, r)
+		tsmFiles = append(tsmFiles, r)
 	}
 
 	// Merge and dedup all the series keys across each reader to reduce
 	// lock contention on the index.
 	keys := make([][]byte, 0, 10000)
 	fieldTypes := make([]influxql.DataType, 0, 10000)
-	merged := merge(readers...)
-	for v := range merged {
-		fieldType := BlockTypeToInfluxQLDataType(v.typ)
+
+	ki := newMergeKeyIterator(tsmFiles, nil)
+	for ki.Next() {
+		key, typ := ki.Read()
+		fieldType := BlockTypeToInfluxQLDataType(typ)
 		if fieldType == influxql.Unknown {
-			return fmt.Errorf("unknown block type: %v", v.typ)
+			return fmt.Errorf("unknown block type: %v", typ)
 		}
 
-		keys = append(keys, v.key)
+		keys = append(keys, key)
 		fieldTypes = append(fieldTypes, fieldType)
 
 		if len(keys) == cap(keys) {
@@ -1118,7 +1180,8 @@ func (e *Engine) readFileFromBackup(tr *tar.Reader, shardRelativePath string, as
 	}
 
 	if asNew {
-		filename = fmt.Sprintf("%09d-%09d.%s", e.FileStore.NextGeneration(), 1, TSMFileExtension)
+		formatFileName := e.GenerateFormatFileNameFunc()
+		filename = formatFileName(e.FileStore.NextGeneration(), 1) + "." + TSMFileExtension
 	}
 
 	tmp := fmt.Sprintf("%s.%s", filepath.Join(e.path, filename), TmpTSMFileExtension)
@@ -1153,7 +1216,7 @@ func (e *Engine) addToIndexFromKey(keys [][]byte, fieldTypes []influxql.DataType
 	for i := 0; i < len(keys); i++ {
 		// Replace tsm key format with index key format.
 		keys[i], field = SeriesAndFieldFromCompositeKey(keys[i])
-		name := tsdb.MeasurementFromSeriesKey(keys[i])
+		name := models.ParseName(keys[i])
 		mf := e.fieldset.CreateFieldsIfNotExists(name)
 		if err := mf.CreateFieldIfNotExists(field, fieldTypes[i]); err != nil {
 			return err
@@ -1181,8 +1244,12 @@ func (e *Engine) addToIndexFromKey(keys [][]byte, fieldTypes []influxql.DataType
 // It returns an error if new points are added to an existing key.
 func (e *Engine) WritePoints(points []models.Point) error {
 	values := make(map[string][]Value, len(points))
-	var keyBuf []byte
-	var baseLen int
+	var (
+		keyBuf    []byte
+		baseLen   int
+		seriesErr error
+	)
+
 	for _, p := range points {
 		keyBuf = append(keyBuf[:0], p.Key()...)
 		keyBuf = append(keyBuf, keyFieldSeparator...)
@@ -1196,6 +1263,37 @@ func (e *Engine) WritePoints(points []models.Point) error {
 			}
 
 			keyBuf = append(keyBuf[:baseLen], iter.FieldKey()...)
+
+			if e.seriesTypeMap != nil {
+				// Fast-path check to see if the field for the series already exists.
+				if v, ok := e.seriesTypeMap.Get(keyBuf); !ok {
+					if typ, err := e.Type(keyBuf); err != nil {
+						// Field type is unknown, we can try to add it.
+					} else if typ != iter.Type() {
+						// Existing type is different from what was passed in, we need to drop
+						// this write and refresh the series type map.
+						seriesErr = tsdb.ErrFieldTypeConflict
+						e.seriesTypeMap.Insert(keyBuf, int(typ))
+						continue
+					}
+
+					// Doesn't exsts, so try to insert
+					vv, ok := e.seriesTypeMap.Insert(keyBuf, int(iter.Type()))
+
+					// We didn't insert and the type that exists isn't what we tried to insert, so
+					// we have a conflict and must drop this field/series.
+					if !ok || vv != int(iter.Type()) {
+						seriesErr = tsdb.ErrFieldTypeConflict
+						continue
+					}
+				} else if v != int(iter.Type()) {
+					// The series already exists, but with a different type.  This is also a type conflict
+					// and we need to drop this field/series.
+					seriesErr = tsdb.ErrFieldTypeConflict
+					continue
+				}
+			}
+
 			var v Value
 			switch iter.Type() {
 			case models.Float:
@@ -1235,17 +1333,28 @@ func (e *Engine) WritePoints(points []models.Point) error {
 	defer e.mu.RUnlock()
 
 	// first try to write to the cache
-	err := e.Cache.WriteMulti(values)
-	if err != nil {
+	if err := e.Cache.WriteMulti(values); err != nil {
 		return err
 	}
 
-	_, err = e.WAL.WriteMulti(values)
-	return err
+	if e.WALEnabled {
+		if _, err := e.WAL.WriteMulti(values); err != nil {
+			return err
+		}
+	}
+	return seriesErr
 }
 
 // DeleteSeriesRange removes the values between min and max (inclusive) from all series
 func (e *Engine) DeleteSeriesRange(itr tsdb.SeriesIterator, min, max int64) error {
+	return e.DeleteSeriesRangeWithPredicate(itr, func(name []byte, tags models.Tags) (int64, int64, bool) {
+		return min, max, true
+	})
+}
+
+// DeleteSeriesRangeWithPredicate removes the values between min and max (inclusive) from all series
+// for which predicate() returns true. If predicate() is nil, then all values in range are removed.
+func (e *Engine) DeleteSeriesRangeWithPredicate(itr tsdb.SeriesIterator, predicate func(name []byte, tags models.Tags) (int64, int64, bool)) error {
 	var disableOnce bool
 
 	// Ensure that the index does not compact away the measurement or series we're
@@ -1262,7 +1371,24 @@ func (e *Engine) DeleteSeriesRange(itr tsdb.SeriesIterator, min, max int64) erro
 		defer fs.Release()
 	}
 
-	var sz int
+	var (
+		sz       int
+		min, max int64 = math.MinInt64, math.MaxInt64
+
+		// Indicator that the min/max time for the current batch has changed and
+		// we need to flush the current batch before appending to it.
+		flushBatch bool
+	)
+
+	// These are reversed from min/max to ensure they are different the first time through.
+	newMin, newMax := int64(math.MaxInt64), int64(math.MinInt64)
+
+	// There is no predicate, so setup newMin/newMax to delete the full time range.
+	if predicate == nil {
+		newMin = min
+		newMax = max
+	}
+
 	batch := make([][]byte, 0, 10000)
 	for {
 		elem, err := itr.Next()
@@ -1272,6 +1398,19 @@ func (e *Engine) DeleteSeriesRange(itr tsdb.SeriesIterator, min, max int64) erro
 			break
 		}
 
+		// See if the series should be deleted and if so, what range of time.
+		if predicate != nil {
+			var shouldDelete bool
+			newMin, newMax, shouldDelete = predicate(elem.Name(), elem.Tags())
+			if !shouldDelete {
+				continue
+			}
+
+			// If the min/max happens to change for the batch, we need to flush
+			// the current batch and start a new one.
+			flushBatch = (min != newMin || max != newMax) && len(batch) > 0
+		}
+
 		if elem.Expr() != nil {
 			if v, ok := elem.Expr().(*influxql.BooleanLiteral); !ok || !v.Val {
 				return errors.New("fields not supported in WHERE clause during deletion")
@@ -1295,18 +1434,23 @@ func (e *Engine) DeleteSeriesRange(itr tsdb.SeriesIterator, min, max int64) erro
 			disableOnce = true
 		}
 
-		key := models.MakeKey(elem.Name(), elem.Tags())
-		sz += len(key)
-		batch = append(batch, key)
-
-		if sz >= deleteFlushThreshold {
+		if sz >= deleteFlushThreshold || flushBatch {
 			// Delete all matching batch.
 			if err := e.deleteSeriesRange(batch, min, max); err != nil {
 				return err
 			}
 			batch = batch[:0]
 			sz = 0
+			flushBatch = false
 		}
+
+		// Use the new min/max time for the next iteration
+		min = newMin
+		max = newMax
+
+		key := models.MakeKey(elem.Name(), elem.Tags())
+		sz += len(key)
+		batch = append(batch, key)
 	}
 
 	if len(batch) > 0 {
@@ -1407,8 +1551,10 @@ func (e *Engine) deleteSeriesRange(seriesKeys [][]byte, min, max int64) error {
 	e.Cache.DeleteRange(deleteKeys, min, max)
 
 	// delete from the WAL
-	if _, err := e.WAL.DeleteRange(deleteKeys, min, max); err != nil {
-		return err
+	if e.WALEnabled {
+		if _, err := e.WAL.DeleteRange(deleteKeys, min, max); err != nil {
+			return err
+		}
 	}
 
 	// The series are deleted on disk, but the index may still say they exist.
@@ -1636,25 +1782,31 @@ func (e *Engine) WriteSnapshot() error {
 		logEnd()
 	}()
 
-	closedFiles, snapshot, err := func() ([]string, *Cache, error) {
+	var formatFileName FormatFileNameFunc
+	closedFiles, snapshot, err := func() (segments []string, snapshot *Cache, err error) {
 		e.mu.Lock()
 		defer e.mu.Unlock()
 
-		if err := e.WAL.CloseSegment(); err != nil {
-			return nil, nil, err
+		if e.WALEnabled {
+			if err = e.WAL.CloseSegment(); err != nil {
+				return
+			}
+
+			segments, err = e.WAL.ClosedSegments()
+			if err != nil {
+				return
+			}
 		}
 
-		segments, err := e.WAL.ClosedSegments()
+		snapshot, err = e.Cache.Snapshot()
 		if err != nil {
-			return nil, nil, err
+			return
 		}
 
-		snapshot, err := e.Cache.Snapshot()
-		if err != nil {
-			return nil, nil, err
-		}
+		// Determine filename generation function under lock.
+		formatFileName = e.GenerateFormatFileNameFunc()
 
-		return segments, snapshot, nil
+		return
 	}()
 
 	if err != nil {
@@ -1675,7 +1827,7 @@ func (e *Engine) WriteSnapshot() error {
 		zap.String("path", e.path),
 		zap.Duration("duration", time.Since(dedup)))
 
-	return e.writeSnapshotAndCommit(log, closedFiles, snapshot)
+	return e.writeSnapshotAndCommit(log, closedFiles, snapshot, formatFileName)
 }
 
 // CreateSnapshot will create a temp directory that holds
@@ -1697,7 +1849,7 @@ func (e *Engine) CreateSnapshot() (string, error) {
 }
 
 // writeSnapshotAndCommit will write the passed cache to a new TSM file and remove the closed WAL segments.
-func (e *Engine) writeSnapshotAndCommit(log *zap.Logger, closedFiles []string, snapshot *Cache) (err error) {
+func (e *Engine) writeSnapshotAndCommit(log *zap.Logger, closedFiles []string, snapshot *Cache, formatFileName FormatFileNameFunc) (err error) {
 	defer func() {
 		if err != nil {
 			e.Cache.ClearSnapshot(false)
@@ -1705,7 +1857,7 @@ func (e *Engine) writeSnapshotAndCommit(log *zap.Logger, closedFiles []string, s
 	}()
 
 	// write the new snapshot files
-	newFiles, err := e.Compactor.WriteSnapshot(snapshot)
+	newFiles, err := e.Compactor.WriteSnapshot(snapshot, formatFileName)
 	if err != nil {
 		log.Info("Error writing snapshot from compactor", zap.Error(err))
 		return err
@@ -1723,8 +1875,10 @@ func (e *Engine) writeSnapshotAndCommit(log *zap.Logger, closedFiles []string, s
 	// clear the snapshot from the in-memory cache, then the old WAL files
 	e.Cache.ClearSnapshot(true)
 
-	if err := e.WAL.Remove(closedFiles); err != nil {
-		log.Info("Error removing closed WAL segments", zap.Error(err))
+	if e.WALEnabled {
+		if err := e.WAL.Remove(closedFiles); err != nil {
+			log.Info("Error removing closed WAL segments", zap.Error(err))
+		}
 	}
 
 	return nil
@@ -1745,7 +1899,7 @@ func (e *Engine) compactCache() {
 
 		case <-t.C:
 			e.Cache.UpdateAge()
-			if e.ShouldCompactCache(e.WAL.LastWriteTime()) {
+			if e.ShouldCompactCache(time.Now()) {
 				start := time.Now()
 				e.traceLogger.Info("Compacting cache", zap.String("path", e.path))
 				err := e.WriteSnapshot()
@@ -1763,15 +1917,18 @@ func (e *Engine) compactCache() {
 
 // ShouldCompactCache returns true if the Cache is over its flush threshold
 // or if the passed in lastWriteTime is older than the write cold threshold.
-func (e *Engine) ShouldCompactCache(lastWriteTime time.Time) bool {
+func (e *Engine) ShouldCompactCache(t time.Time) bool {
 	sz := e.Cache.Size()
 
 	if sz == 0 {
 		return false
 	}
 
-	return sz > e.CacheFlushMemorySizeThreshold ||
-		time.Since(lastWriteTime) > e.CacheFlushWriteColdDuration
+	if sz > e.CacheFlushMemorySizeThreshold {
+		return true
+	}
+
+	return t.Sub(e.Cache.LastWriteTime()) > e.CacheFlushWriteColdDuration
 }
 
 func (e *Engine) compact(wg *sync.WaitGroup) {
@@ -1793,7 +1950,7 @@ func (e *Engine) compact(wg *sync.WaitGroup) {
 			level1Groups := e.CompactionPlan.PlanLevel(1)
 			level2Groups := e.CompactionPlan.PlanLevel(2)
 			level3Groups := e.CompactionPlan.PlanLevel(3)
-			level4Groups := e.CompactionPlan.Plan(e.WAL.LastWriteTime())
+			level4Groups := e.CompactionPlan.Plan(e.FileStore.LastModified())
 			atomic.StoreInt64(&e.stats.TSMOptimizeCompactionsQueue, int64(len(level4Groups)))
 
 			// If no full compactions are need, see if an optimize is needed
@@ -1952,11 +2109,10 @@ func (s *compactionStrategy) Apply() {
 // compactGroup executes the compaction strategy against a single CompactionGroup.
 func (s *compactionStrategy) compactGroup() {
 	group := s.group
-	start := time.Now()
 	log, logEnd := logger.NewOperation(s.logger, "TSM compaction", "tsm1_compact_group")
 	defer logEnd()
 
-	log.Info("Beginning compaction", zap.Int("tsm1_files", len(group)))
+	log.Info("Beginning compaction", zap.Int("tsm1_files_n", len(group)))
 	for i, f := range group {
 		log.Info("Compacting file", zap.Int("tsm1_index", i), zap.String("tsm1_file", f))
 	}
@@ -2000,9 +2156,7 @@ func (s *compactionStrategy) compactGroup() {
 		log.Info("Compacted file", zap.Int("tsm1_index", i), zap.String("tsm1_file", f))
 	}
 	log.Info("Finished compacting files",
-		zap.Int("groups", len(group)),
-		zap.Int("files", len(files)),
-		zap.Duration("duration", time.Since(start)))
+		zap.Int("tsm1_files_n", len(files)))
 	atomic.AddInt64(s.successStat, 1)
 }
 
@@ -2793,6 +2947,32 @@ func (e *Engine) IteratorCost(measurement string, opt query.IteratorOptions) (qu
 	return cost, nil
 }
 
+// Type returns FieldType for a series.  If the series does not
+// exist, ErrUnkownFieldType is returned.
+func (e *Engine) Type(series []byte) (models.FieldType, error) {
+	if typ, err := e.Cache.Type(series); err == nil {
+		return typ, nil
+	}
+
+	typ, err := e.FileStore.Type(series)
+	if err != nil {
+		return 0, err
+	}
+	switch typ {
+	case BlockFloat64:
+		return models.Float, nil
+	case BlockInteger:
+		return models.Integer, nil
+	case BlockUnsigned:
+		return models.Unsigned, nil
+	case BlockString:
+		return models.String, nil
+	case BlockBoolean:
+		return models.Boolean, nil
+	}
+	return 0, tsdb.ErrUnknownFieldType
+}
+
 func (e *Engine) seriesCost(seriesKey, field string, tmin, tmax int64) query.IteratorCost {
 	key := SeriesFieldKeyBytes(seriesKey, field)
 	c := e.FileStore.Cost(key, tmin, tmax)
@@ -2817,22 +2997,19 @@ func SeriesFieldKeyBytes(seriesKey, field string) []byte {
 }
 
 var (
-	blockToFieldType = []influxql.DataType{
+	blockToFieldType = [8]influxql.DataType{
 		BlockFloat64:  influxql.Float,
 		BlockInteger:  influxql.Integer,
 		BlockBoolean:  influxql.Boolean,
 		BlockString:   influxql.String,
 		BlockUnsigned: influxql.Unsigned,
+		5:             influxql.Unknown,
+		6:             influxql.Unknown,
+		7:             influxql.Unknown,
 	}
 )
 
-func BlockTypeToInfluxQLDataType(typ byte) influxql.DataType {
-	if int(typ) < len(blockToFieldType) {
-		return blockToFieldType[typ]
-	}
-
-	return influxql.Unknown
-}
+func BlockTypeToInfluxQLDataType(typ byte) influxql.DataType { return blockToFieldType[typ&7] }
 
 // SeriesAndFieldFromCompositeKey returns the series key and the field key extracted from the composite key.
 func SeriesAndFieldFromCompositeKey(key []byte) ([]byte, []byte) {
