@@ -37,6 +37,7 @@ type Index interface {
 	MeasurementsSketches() (estimator.Sketch, estimator.Sketch, error)
 	SeriesN() int64
 	SeriesSketches() (estimator.Sketch, estimator.Sketch, error)
+	SeriesIDSet() *SeriesIDSet
 
 	HasTagKey(name, key []byte) (bool, error)
 	HasTagValue(name, key, value []byte) (bool, error)
@@ -60,10 +61,16 @@ type Index interface {
 	// Size of the index on disk, if applicable.
 	DiskSizeBytes() int64
 
+	// Bytes estimates the memory footprint of this Index, in bytes.
+	Bytes() int
+
 	// To be removed w/ tsi1.
 	SetFieldName(measurement []byte, name string)
 
 	Type() string
+	// Returns a unique reference ID to the index instance.
+	// For inmem, returns a reference to the backing Index, not ShardIndex.
+	UniqueReferenceID() uintptr
 
 	Rebuild()
 }
@@ -157,6 +164,53 @@ type SeriesIDIterator interface {
 	Close() error
 }
 
+// SeriesIDSetIterator represents an iterator that can produce a SeriesIDSet.
+type SeriesIDSetIterator interface {
+	SeriesIDIterator
+	SeriesIDSet() *SeriesIDSet
+}
+
+type seriesIDSetIterator struct {
+	ss  *SeriesIDSet
+	itr SeriesIDSetIterable
+}
+
+func NewSeriesIDSetIterator(ss *SeriesIDSet) SeriesIDSetIterator {
+	if ss == nil || ss.bitmap == nil {
+		return nil
+	}
+	return &seriesIDSetIterator{ss: ss, itr: ss.Iterator()}
+}
+
+func (itr *seriesIDSetIterator) Next() (SeriesIDElem, error) {
+	if !itr.itr.HasNext() {
+		return SeriesIDElem{}, nil
+	}
+	return SeriesIDElem{SeriesID: uint64(itr.itr.Next())}, nil
+}
+
+func (itr *seriesIDSetIterator) Close() error { return nil }
+
+func (itr *seriesIDSetIterator) SeriesIDSet() *SeriesIDSet { return itr.ss }
+
+// NewSeriesIDSetIterators returns a slice of SeriesIDSetIterator if all itrs
+// can be type casted. Otherwise returns nil.
+func NewSeriesIDSetIterators(itrs []SeriesIDIterator) []SeriesIDSetIterator {
+	if len(itrs) == 0 {
+		return nil
+	}
+
+	a := make([]SeriesIDSetIterator, len(itrs))
+	for i := range itrs {
+		if itr, ok := itrs[i].(SeriesIDSetIterator); ok {
+			a[i] = itr
+		} else {
+			return nil
+		}
+	}
+	return a
+}
+
 // ReadAllSeriesIDIterator returns all ids from the iterator.
 func ReadAllSeriesIDIterator(itr SeriesIDIterator) ([]uint64, error) {
 	if itr == nil {
@@ -198,6 +252,15 @@ func (itr *SeriesIDSliceIterator) Next() (SeriesIDElem, error) {
 
 func (itr *SeriesIDSliceIterator) Close() error { return nil }
 
+// SeriesIDSet returns a set of all remaining ids.
+func (itr *SeriesIDSliceIterator) SeriesIDSet() *SeriesIDSet {
+	s := NewSeriesIDSet()
+	for _, id := range itr.ids {
+		s.AddNoLock(id)
+	}
+	return s
+}
+
 type SeriesIDIterators []SeriesIDIterator
 
 func (a SeriesIDIterators) Close() (err error) {
@@ -352,6 +415,19 @@ func MergeSeriesIDIterators(itrs ...SeriesIDIterator) SeriesIDIterator {
 		return itrs[0]
 	}
 
+	// Merge as series id sets, if available.
+	if a := NewSeriesIDSetIterators(itrs); a != nil {
+		sets := make([]*SeriesIDSet, len(a))
+		for i := range a {
+			sets[i] = a[i].SeriesIDSet()
+		}
+
+		ss := NewSeriesIDSet()
+		ss.Merge(sets...)
+		SeriesIDIterators(itrs).Close()
+		return NewSeriesIDSetIterator(ss)
+	}
+
 	return &seriesIDMergeIterator{
 		buf:  make([]SeriesIDElem, len(itrs)),
 		itrs: itrs,
@@ -420,6 +496,13 @@ func IntersectSeriesIDIterators(itr0, itr1 SeriesIDIterator) SeriesIDIterator {
 		return nil
 	}
 
+	// Create series id set, if available.
+	if a := NewSeriesIDSetIterators([]SeriesIDIterator{itr0, itr1}); a != nil {
+		itr0.Close()
+		itr1.Close()
+		return NewSeriesIDSetIterator(a[0].SeriesIDSet().And(a[1].SeriesIDSet()))
+	}
+
 	return &seriesIDIntersectIterator{itrs: [2]SeriesIDIterator{itr0, itr1}}
 }
 
@@ -502,6 +585,15 @@ func UnionSeriesIDIterators(itr0, itr1 SeriesIDIterator) SeriesIDIterator {
 		return itr0
 	}
 
+	// Create series id set, if available.
+	if a := NewSeriesIDSetIterators([]SeriesIDIterator{itr0, itr1}); a != nil {
+		itr0.Close()
+		itr1.Close()
+		ss := NewSeriesIDSet()
+		ss.Merge(a[0].SeriesIDSet(), a[1].SeriesIDSet())
+		return NewSeriesIDSetIterator(ss)
+	}
+
 	return &seriesIDUnionIterator{itrs: [2]SeriesIDIterator{itr0, itr1}}
 }
 
@@ -579,6 +671,14 @@ func DifferenceSeriesIDIterators(itr0, itr1 SeriesIDIterator) SeriesIDIterator {
 		itr1.Close()
 		return nil
 	}
+
+	// Create series id set, if available.
+	if a := NewSeriesIDSetIterators([]SeriesIDIterator{itr0, itr1}); a != nil {
+		itr0.Close()
+		itr1.Close()
+		return NewSeriesIDSetIterator(a[0].SeriesIDSet().AndNot(a[1].SeriesIDSet()))
+	}
+
 	return &seriesIDDifferenceIterator{itrs: [2]SeriesIDIterator{itr0, itr1}}
 }
 
@@ -752,7 +852,7 @@ func (itr *seriesPointIterator) readSeriesKeys(name []byte) error {
 
 	// Slurp all series keys.
 	itr.keys = itr.keys[:0]
-	for {
+	for i := 0; ; i++ {
 		elem, err := sitr.Next()
 		if err != nil {
 			return err
@@ -760,6 +860,15 @@ func (itr *seriesPointIterator) readSeriesKeys(name []byte) error {
 			break
 		}
 
+		// Periodically check for interrupt.
+		if i&0xFF == 0xFF {
+			select {
+			case <-itr.opt.InterruptCh:
+				return itr.Close()
+			default:
+			}
+		}
+
 		key := itr.indexSet.SeriesFile.SeriesKey(elem.SeriesID)
 		if len(key) == 0 {
 			continue
@@ -1088,13 +1197,23 @@ func (itr *tagValueMergeIterator) Next() (_ []byte, err error) {
 	return value, nil
 }
 
-// IndexSet represents a list of indexes.
+// IndexSet represents a list of indexes, all belonging to one database.
 type IndexSet struct {
 	Indexes    []Index                // The set of indexes comprising this IndexSet.
 	SeriesFile *SeriesFile            // The Series File associated with the db for this set.
 	fieldSets  []*MeasurementFieldSet // field sets for _all_ indexes in this set's DB.
 }
 
+// HasInmemIndex returns true if any in-memory index is in use.
+func (is IndexSet) HasInmemIndex() bool {
+	for _, idx := range is.Indexes {
+		if idx.Type() == "inmem" {
+			return true
+		}
+	}
+	return false
+}
+
 // Database returns the database name of the first index.
 func (is IndexSet) Database() string {
 	if len(is.Indexes) == 0 {
@@ -1126,7 +1245,8 @@ func (is IndexSet) HasField(measurement []byte, field string) bool {
 	return false
 }
 
-// DedupeInmemIndexes returns an index set which removes duplicate in-memory indexes.
+// DedupeInmemIndexes returns an index set which removes duplicate indexes.
+// Useful because inmem indexes are shared by shards per database.
 func (is IndexSet) DedupeInmemIndexes() IndexSet {
 	other := IndexSet{
 		Indexes:    make([]Index, 0, len(is.Indexes)),
@@ -1134,18 +1254,16 @@ func (is IndexSet) DedupeInmemIndexes() IndexSet {
 		fieldSets:  make([]*MeasurementFieldSet, 0, len(is.Indexes)),
 	}
 
-	var hasInmem bool
+	uniqueIndexes := make(map[uintptr]Index)
 	for _, idx := range is.Indexes {
-		other.fieldSets = append(other.fieldSets, idx.FieldSet())
-		if idx.Type() == "inmem" {
-			if !hasInmem {
-				other.Indexes = append(other.Indexes, idx)
-				hasInmem = true
-			}
-			continue
-		}
-		other.Indexes = append(other.Indexes, idx)
+		uniqueIndexes[idx.UniqueReferenceID()] = idx
 	}
+
+	for _, idx := range uniqueIndexes {
+		other.Indexes = append(other.Indexes, idx)
+		other.fieldSets = append(other.fieldSets, idx.FieldSet())
+	}
+
 	return other
 }
 
@@ -1346,7 +1464,6 @@ func (is IndexSet) measurementNamesByTagFilter(auth query.Authorizer, op influxq
 				}
 				if !valEqual(ve) {
 					continue
-
 				}
 
 				tagMatch = true
@@ -1363,6 +1480,7 @@ func (is IndexSet) measurementNamesByTagFilter(auth query.Authorizer, op influxq
 					continue
 				}
 				defer sitr.Close()
+				sitr = FilterUndeletedSeriesIDIterator(is.SeriesFile, sitr)
 
 				// Locate a series with this matching tag value that's authorized.
 				for {
@@ -1431,6 +1549,7 @@ func (is IndexSet) measurementAuthorizedSeries(auth query.Authorizer, name []byt
 		return false
 	}
 	defer sitr.Close()
+	sitr = FilterUndeletedSeriesIDIterator(is.SeriesFile, sitr)
 
 	for {
 		series, err := sitr.Next()
@@ -1547,6 +1666,10 @@ func (is IndexSet) tagValueIterator(name, key []byte) (TagValueIterator, error)
 // TagKeyHasAuthorizedSeries determines if there exists an authorized series for
 // the provided measurement name and tag key.
 func (is IndexSet) TagKeyHasAuthorizedSeries(auth query.Authorizer, name, tagKey []byte) (bool, error) {
+	if !is.HasInmemIndex() && query.AuthorizerIsOpen(auth) {
+		return true, nil
+	}
+
 	release := is.SeriesFile.Retain()
 	defer release()
 
@@ -1557,6 +1680,7 @@ func (is IndexSet) TagKeyHasAuthorizedSeries(auth query.Authorizer, name, tagKey
 		return false, nil
 	}
 	defer itr.Close()
+	itr = FilterUndeletedSeriesIDIterator(is.SeriesFile, itr)
 
 	for {
 		e, err := itr.Next()
@@ -1584,7 +1708,12 @@ func (is IndexSet) TagKeyHasAuthorizedSeries(auth query.Authorizer, name, tagKey
 func (is IndexSet) MeasurementSeriesIDIterator(name []byte) (SeriesIDIterator, error) {
 	release := is.SeriesFile.Retain()
 	defer release()
-	return is.measurementSeriesIDIterator(name)
+
+	itr, err := is.measurementSeriesIDIterator(name)
+	if err != nil {
+		return nil, err
+	}
+	return FilterUndeletedSeriesIDIterator(is.SeriesFile, itr), nil
 }
 
 // measurementSeriesIDIterator does not provide any locking on the Series file.
@@ -1601,7 +1730,7 @@ func (is IndexSet) measurementSeriesIDIterator(name []byte) (SeriesIDIterator, e
 			a = append(a, itr)
 		}
 	}
-	return FilterUndeletedSeriesIDIterator(is.SeriesFile, MergeSeriesIDIterators(a...)), nil
+	return MergeSeriesIDIterators(a...), nil
 }
 
 // ForEachMeasurementTagKey iterates over all tag keys in a measurement and applies
@@ -1654,7 +1783,12 @@ func (is IndexSet) MeasurementTagKeysByExpr(name []byte, expr influxql.Expr) (ma
 func (is IndexSet) TagKeySeriesIDIterator(name, key []byte) (SeriesIDIterator, error) {
 	release := is.SeriesFile.Retain()
 	defer release()
-	return is.tagKeySeriesIDIterator(name, key)
+
+	itr, err := is.tagKeySeriesIDIterator(name, key)
+	if err != nil {
+		return nil, err
+	}
+	return FilterUndeletedSeriesIDIterator(is.SeriesFile, itr), nil
 }
 
 // tagKeySeriesIDIterator returns a series iterator for all values across a
@@ -1672,14 +1806,19 @@ func (is IndexSet) tagKeySeriesIDIterator(name, key []byte) (SeriesIDIterator, e
 			a = append(a, itr)
 		}
 	}
-	return FilterUndeletedSeriesIDIterator(is.SeriesFile, MergeSeriesIDIterators(a...)), nil
+	return MergeSeriesIDIterators(a...), nil
 }
 
 // TagValueSeriesIDIterator returns a series iterator for a single tag value.
 func (is IndexSet) TagValueSeriesIDIterator(name, key, value []byte) (SeriesIDIterator, error) {
 	release := is.SeriesFile.Retain()
 	defer release()
-	return is.tagValueSeriesIDIterator(name, key, value)
+
+	itr, err := is.tagValueSeriesIDIterator(name, key, value)
+	if err != nil {
+		return nil, err
+	}
+	return FilterUndeletedSeriesIDIterator(is.SeriesFile, itr), nil
 }
 
 // tagValueSeriesIDIterator does not provide any locking on the Series File.
@@ -1696,7 +1835,7 @@ func (is IndexSet) tagValueSeriesIDIterator(name, key, value []byte) (SeriesIDIt
 			a = append(a, itr)
 		}
 	}
-	return FilterUndeletedSeriesIDIterator(is.SeriesFile, MergeSeriesIDIterators(a...)), nil
+	return MergeSeriesIDIterators(a...), nil
 }
 
 // MeasurementSeriesByExprIterator returns a series iterator for a measurement
@@ -1716,7 +1855,11 @@ func (is IndexSet) MeasurementSeriesByExprIterator(name []byte, expr influxql.Ex
 func (is IndexSet) measurementSeriesByExprIterator(name []byte, expr influxql.Expr) (SeriesIDIterator, error) {
 	// Return all series for the measurement if there are no tag expressions.
 	if expr == nil {
-		return is.measurementSeriesIDIterator(name)
+		itr, err := is.measurementSeriesIDIterator(name)
+		if err != nil {
+			return nil, err
+		}
+		return FilterUndeletedSeriesIDIterator(is.SeriesFile, itr), nil
 	}
 
 	itr, err := is.seriesByExprIterator(name, expr)
@@ -1740,6 +1883,8 @@ func (is IndexSet) MeasurementSeriesKeysByExpr(name []byte, expr influxql.Expr)
 	}
 	defer itr.Close()
 
+	// measurementSeriesByExprIterator filters deleted series; no need to do so here.
+
 	// Iterate over all series and generate keys.
 	var keys [][]byte
 	for {
@@ -1842,7 +1987,13 @@ func (is IndexSet) seriesByBinaryExprIterator(name []byte, n *influxql.BinaryExp
 	if !ok {
 		key, ok = n.RHS.(*influxql.VarRef)
 		if !ok {
-			return nil, fmt.Errorf("invalid expression: %s", n.String())
+			// This is an expression we do not know how to evaluate. Let the
+			// query engine take care of this.
+			itr, err := is.measurementSeriesIDIterator(name)
+			if err != nil {
+				return nil, err
+			}
+			return newSeriesIDExprIterator(itr, n), nil
 		}
 		value = n.LHS
 	}
@@ -1874,10 +2025,13 @@ func (is IndexSet) seriesByBinaryExprIterator(name []byte, n *influxql.BinaryExp
 	case *influxql.VarRef:
 		return is.seriesByBinaryExprVarRefIterator(name, []byte(key.Val), value, n.Op)
 	default:
-		if n.Op == influxql.NEQ || n.Op == influxql.NEQREGEX {
-			return is.measurementSeriesIDIterator(name)
+		// We do not know how to evaluate this expression so pass it
+		// on to the query engine.
+		itr, err := is.measurementSeriesIDIterator(name)
+		if err != nil {
+			return nil, err
 		}
-		return nil, nil
+		return newSeriesIDExprIterator(itr, n), nil
 	}
 }
 
@@ -1976,7 +2130,11 @@ func (is IndexSet) seriesByBinaryExprVarRefIterator(name, key []byte, value *inf
 func (is IndexSet) MatchTagValueSeriesIDIterator(name, key []byte, value *regexp.Regexp, matches bool) (SeriesIDIterator, error) {
 	release := is.SeriesFile.Retain()
 	defer release()
-	return is.matchTagValueSeriesIDIterator(name, key, value, matches)
+	itr, err := is.matchTagValueSeriesIDIterator(name, key, value, matches)
+	if err != nil {
+		return nil, err
+	}
+	return FilterUndeletedSeriesIDIterator(is.SeriesFile, itr), nil
 }
 
 // matchTagValueSeriesIDIterator returns a series iterator for tags which match
@@ -2280,6 +2438,7 @@ func (is IndexSet) MeasurementTagKeyValuesByExpr(auth query.Authorizer, name []b
 					continue
 				}
 				defer sitr.Close()
+				sitr = FilterUndeletedSeriesIDIterator(is.SeriesFile, sitr)
 
 				for {
 					se, err := sitr.Next()
@@ -2338,6 +2497,8 @@ func (is IndexSet) TagSets(sfile *SeriesFile, name []byte, opt query.IteratorOpt
 		return nil, nil
 	}
 	defer itr.Close()
+	// measurementSeriesByExprIterator filters deleted series IDs; no need to
+	// do so here.
 
 	var dims []string
 	if len(opt.Dimensions) > 0 {
