@@ -11,6 +11,7 @@ import (
 	"strconv"
 	"sync"
 	"sync/atomic"
+	"unsafe"
 
 	"github.com/cespare/xxhash"
 	"github.com/influxdata/influxdb/models"
@@ -85,6 +86,30 @@ var WithMaximumLogFileSize = func(size int64) IndexOption {
 	}
 }
 
+// DisableFsync disables flushing and syncing of underlying files. Primarily this
+// impacts the LogFiles. This option can be set when working with the index in
+// an offline manner, for cases where a hard failure can be overcome by re-running the tooling.
+var DisableFsync = func() IndexOption {
+	return func(i *Index) {
+		i.disableFsync = true
+	}
+}
+
+// WithLogFileBufferSize sets the size of the buffer used within LogFiles.
+// Typically appending an entry to a LogFile involves writing 11 or 12 bytes, so
+// depending on how many new series are being created within a batch, it may
+// be appropriate to set this.
+var WithLogFileBufferSize = func(sz int) IndexOption {
+	return func(i *Index) {
+		if sz > 1<<17 { // 128K
+			sz = 1 << 17
+		} else if sz < 1<<12 {
+			sz = 1 << 12 // 4K (runtime default)
+		}
+		i.logfileBufferSize = sz
+	}
+}
+
 // Index represents a collection of layered index files and WAL.
 type Index struct {
 	mu         sync.RWMutex
@@ -95,6 +120,8 @@ type Index struct {
 	path               string      // Root directory of the index partitions.
 	disableCompactions bool        // Initially disables compactions on the index.
 	maxLogFileSize     int64       // Maximum size of a LogFile before it's compacted.
+	logfileBufferSize  int         // The size of the buffer used by the LogFile.
+	disableFsync       bool        // Disables flushing buffers and fsyning files. Used when working with indexes offline.
 	logger             *zap.Logger // Index's logger.
 
 	// The following must be set when initializing an Index.
@@ -108,6 +135,10 @@ type Index struct {
 	PartitionN uint64
 }
 
+func (i *Index) UniqueReferenceID() uintptr {
+	return uintptr(unsafe.Pointer(i))
+}
+
 // NewIndex returns a new instance of Index.
 func NewIndex(sfile *tsdb.SeriesFile, database string, options ...IndexOption) *Index {
 	idx := &Index{
@@ -126,6 +157,29 @@ func NewIndex(sfile *tsdb.SeriesFile, database string, options ...IndexOption) *
 	return idx
 }
 
+// Bytes estimates the memory footprint of this Index, in bytes.
+func (i *Index) Bytes() int {
+	var b int
+	i.mu.RLock()
+	b += 24 // mu RWMutex is 24 bytes
+	b += int(unsafe.Sizeof(i.partitions))
+	for _, p := range i.partitions {
+		b += int(unsafe.Sizeof(p)) + p.bytes()
+	}
+	b += int(unsafe.Sizeof(i.opened))
+	b += int(unsafe.Sizeof(i.path)) + len(i.path)
+	b += int(unsafe.Sizeof(i.disableCompactions))
+	b += int(unsafe.Sizeof(i.maxLogFileSize))
+	b += int(unsafe.Sizeof(i.logger))
+	b += int(unsafe.Sizeof(i.sfile))
+	// Do not count SeriesFile because it belongs to the code that constructed this Index.
+	b += int(unsafe.Sizeof(i.database)) + len(i.database)
+	b += int(unsafe.Sizeof(i.version))
+	b += int(unsafe.Sizeof(i.PartitionN))
+	i.mu.RUnlock()
+	return b
+}
+
 // Database returns the name of the database the index was initialized with.
 func (i *Index) Database() string {
 	return i.database
@@ -178,7 +232,8 @@ func (i *Index) Open() error {
 	for j := 0; j < len(i.partitions); j++ {
 		p := NewPartition(i.sfile, filepath.Join(i.path, fmt.Sprint(j)))
 		p.MaxLogFileSize = i.maxLogFileSize
-		p.Database = i.database
+		p.nosync = i.disableFsync
+		p.logbufferSize = i.logfileBufferSize
 		p.logger = i.logger.With(zap.String("tsi1_partition", fmt.Sprint(j+1)))
 		i.partitions[j] = p
 	}
@@ -864,7 +919,7 @@ func (i *Index) RetainFileSet() (*FileSet, error) {
 	i.mu.RLock()
 	defer i.mu.RUnlock()
 
-	fs, _ := NewFileSet(i.database, nil, i.sfile, nil)
+	fs, _ := NewFileSet(nil, i.sfile, nil)
 	for _, p := range i.partitions {
 		pfs, err := p.RetainFileSet()
 		if err != nil {
