@@ -6,6 +6,7 @@ import (
 	"errors"
 	"io"
 	"sort"
+	"unsafe"
 
 	"github.com/influxdata/influxdb/pkg/estimator"
 	"github.com/influxdata/influxdb/pkg/estimator/hll"
@@ -61,6 +62,16 @@ type MeasurementBlock struct {
 	version int // block version
 }
 
+// bytes estimates the memory footprint of this MeasurementBlock, in bytes.
+func (blk *MeasurementBlock) bytes() int {
+	var b int
+	// Do not count contents of blk.data or blk.hashData because they reference into an external []byte
+	b += blk.sketch.Bytes()
+	b += blk.tSketch.Bytes()
+	b += int(unsafe.Sizeof(*blk))
+	return b
+}
+
 // Version returns the encoding version parsed from the data.
 // Only valid after UnmarshalBinary() has been successfully invoked.
 func (blk *MeasurementBlock) Version() int { return blk.version }
@@ -191,7 +202,10 @@ func (itr *rawSeriesIDIterator) Next() (tsdb.SeriesIDElem, error) {
 		return tsdb.SeriesIDElem{}, nil
 	}
 
-	delta, n := binary.Uvarint(itr.data)
+	delta, n, err := uvarint(itr.data)
+	if err != nil {
+		return tsdb.SeriesIDElem{}, err
+	}
 	itr.data = itr.data[n:]
 
 	seriesID := itr.prev + uint64(delta)
@@ -199,6 +213,22 @@ func (itr *rawSeriesIDIterator) Next() (tsdb.SeriesIDElem, error) {
 	return tsdb.SeriesIDElem{SeriesID: seriesID}, nil
 }
 
+func (itr *rawSeriesIDIterator) SeriesIDSet() *tsdb.SeriesIDSet {
+	ss := tsdb.NewSeriesIDSet()
+	for data, prev := itr.data, uint64(0); len(data) > 0; {
+		delta, n, err := uvarint(data)
+		if err != nil {
+			break
+		}
+		data = data[n:]
+
+		seriesID := prev + uint64(delta)
+		prev = seriesID
+		ss.AddNoLock(seriesID)
+	}
+	return ss
+}
+
 // MeasurementBlockTrailer represents meta data at the end of a MeasurementBlock.
 type MeasurementBlockTrailer struct {
 	Version int // Encoding version
@@ -360,11 +390,14 @@ func (e *MeasurementBlockElem) SeriesIDs() []uint64 {
 func (e *MeasurementBlockElem) ForEachSeriesID(fn func(uint64) error) error {
 	var prev uint64
 	for data := e.series.data; len(data) > 0; {
-		delta, n := binary.Uvarint(data)
+		delta, n, err := uvarint(data)
+		if err != nil {
+			return err
+		}
 		data = data[n:]
 
 		seriesID := prev + uint64(delta)
-		if err := fn(seriesID); err != nil {
+		if err = fn(seriesID); err != nil {
 			return err
 		}
 		prev = seriesID
@@ -387,13 +420,22 @@ func (e *MeasurementBlockElem) UnmarshalBinary(data []byte) error {
 	e.tagBlock.size, data = int64(binary.BigEndian.Uint64(data)), data[8:]
 
 	// Parse name.
-	sz, n := binary.Uvarint(data)
+	sz, n, err := uvarint(data)
+	if err != nil {
+		return err
+	}
 	e.name, data = data[n:n+int(sz)], data[n+int(sz):]
 
 	// Parse series data.
-	v, n := binary.Uvarint(data)
+	v, n, err := uvarint(data)
+	if err != nil {
+		return err
+	}
 	e.series.n, data = uint64(v), data[n:]
-	sz, n = binary.Uvarint(data)
+	sz, n, err = uvarint(data)
+	if err != nil {
+		return err
+	}
 	data = data[n:]
 	e.series.data, data = data[:sz], data[sz:]
 
