@@ -12,6 +12,7 @@ import (
 	"strings"
 	"sync"
 	"time"
+	"unsafe"
 
 	"github.com/influxdata/influxdb/logger"
 	"github.com/influxdata/influxdb/models"
@@ -62,15 +63,14 @@ type Partition struct {
 	// Fieldset shared with engine.
 	fieldset *tsdb.MeasurementFieldSet
 
-	// Name of database.
-	Database string
-
 	// Directory of the Partition's index files.
 	path string
 	id   string // id portion of path.
 
 	// Log file compaction thresholds.
 	MaxLogFileSize int64
+	nosync         bool // when true, flushing and syncing of LogFile will be disabled.
+	logbufferSize  int  // the LogFile's buffer is set to this value.
 
 	// Frequency of compaction checks.
 	compactionInterrupt chan struct{}
@@ -93,7 +93,6 @@ func NewPartition(sfile *tsdb.SeriesFile, path string) *Partition {
 		sfile:       sfile,
 		seriesIDSet: tsdb.NewSeriesIDSet(),
 
-		// Default compaction thresholds.
 		MaxLogFileSize: tsdb.DefaultMaxIndexLogFileSize,
 
 		// compactionEnabled: true,
@@ -104,6 +103,39 @@ func NewPartition(sfile *tsdb.SeriesFile, path string) *Partition {
 	}
 }
 
+// bytes estimates the memory footprint of this Partition, in bytes.
+func (i *Partition) bytes() int {
+	var b int
+	b += 24 // mu RWMutex is 24 bytes
+	b += int(unsafe.Sizeof(i.opened))
+	// Do not count SeriesFile because it belongs to the code that constructed this Partition.
+	b += int(unsafe.Sizeof(i.activeLogFile)) + i.activeLogFile.bytes()
+	b += int(unsafe.Sizeof(i.fileSet)) + i.fileSet.bytes()
+	b += int(unsafe.Sizeof(i.seq))
+	b += int(unsafe.Sizeof(i.seriesIDSet)) + i.seriesIDSet.Bytes()
+	b += int(unsafe.Sizeof(i.levels))
+	for _, level := range i.levels {
+		b += int(unsafe.Sizeof(level))
+	}
+	b += int(unsafe.Sizeof(i.levelCompacting))
+	for _, levelCompacting := range i.levelCompacting {
+		b += int(unsafe.Sizeof(levelCompacting))
+	}
+	b += 12 // once sync.Once is 12 bytes
+	b += int(unsafe.Sizeof(i.closing))
+	b += 16 // wg sync.WaitGroup is 16 bytes
+	b += int(unsafe.Sizeof(i.fieldset)) + i.fieldset.Bytes()
+	b += int(unsafe.Sizeof(i.path)) + len(i.path)
+	b += int(unsafe.Sizeof(i.id)) + len(i.id)
+	b += int(unsafe.Sizeof(i.MaxLogFileSize))
+	b += int(unsafe.Sizeof(i.compactionInterrupt))
+	b += int(unsafe.Sizeof(i.compactionsDisabled))
+	b += int(unsafe.Sizeof(i.logger))
+	b += int(unsafe.Sizeof(i.manifestSize))
+	b += int(unsafe.Sizeof(i.version))
+	return b
+}
+
 // ErrIncompatibleVersion is returned when attempting to read from an
 // incompatible tsi1 manifest file.
 var ErrIncompatibleVersion = errors.New("incompatible tsi1 index MANIFEST")
@@ -178,7 +210,7 @@ func (i *Partition) Open() error {
 			files = append(files, f)
 		}
 	}
-	fs, err := NewFileSet(i.Database, i.levels, i.sfile, files)
+	fs, err := NewFileSet(i.levels, i.sfile, files)
 	if err != nil {
 		return err
 	}
@@ -216,6 +248,9 @@ func (i *Partition) Open() error {
 // openLogFile opens a log file and appends it to the index.
 func (i *Partition) openLogFile(path string) (*LogFile, error) {
 	f := NewLogFile(i.sfile, path)
+	f.nosync = i.nosync
+	f.bufferSize = i.logbufferSize
+
 	if err := f.Open(); err != nil {
 		return nil, err
 	}
